from audiosample import AudioSample
import numpy as np
def test_stream_wav_to_mp3():
    audio_data = AudioSample().beep(100)
    audio_data_wav = audio_data.as_data(force_out_format='s16le')
    print(f"audio_data_wav: {len(audio_data_wav)=}")
    def chunk_data(data):
        CHUNK_SIZE = 1000
        for i in range(0, len(data), CHUNK_SIZE):
            # print(".", end="")
            yield data[i:i+CHUNK_SIZE]

    orig = AudioSample(audio_data_wav, force_read_format='s16le', force_read_sample_rate=48000, force_read_channels=1).as_data(force_out_format='mp3')
    au = AudioSample(chunk_data(audio_data_wav),
                     force_read_format='s16le', force_read_sample_rate=48000, force_read_channels=1)
    tmp = au.as_data(force_out_format='mp3')
    assert len(tmp) == len(orig)

    au = AudioSample(chunk_data(audio_data_wav),
                     force_read_format='s16le', force_read_sample_rate=48000, force_read_channels=1)


    tmp2 = b''
    for i,j in enumerate(au.as_data_stream(force_out_format='mp3')):
       tmp2 += j
    assert tmp2 == tmp
    print(f"tmp: {len(tmp)=}")


def test_stream_mp3_to_ogg():
    audio_data = AudioSample().beep(100)
    audio_data_mp3 = audio_data.as_data(force_out_format='mp3')
    print(f"audio_data_mp3: {len(audio_data_mp3)=}")
    def chunk_data(data):
        CHUNK_SIZE = 1000
        for i in range(0, len(data), CHUNK_SIZE):
            # print(".", end="")
            yield data[i:i+CHUNK_SIZE]

    orig = AudioSample(audio_data_mp3, force_read_format='mp3').as_data(force_out_format='opus')

    au = AudioSample(chunk_data(audio_data_mp3))
    tmp = au.as_data(force_out_format='opus')
    au = AudioSample(chunk_data(audio_data_mp3))
    tmp2 = b''
    for i,j in enumerate(au.as_data_stream(force_out_format='opus')):
       tmp2 += j
    assert len(tmp2) == len(tmp)
    assert np.all(AudioSample(tmp2).as_numpy() == AudioSample(tmp).as_numpy())
    # assert tmp2 == tmp

def test_wav_to_mulaw():
    audio_data = AudioSample().beep(100)
    audio_data_wav = audio_data.as_data(force_out_format='s16le')
    print(f"audio_data_wav: {len(audio_data_wav)=}")
    def chunk_data(data):
        CHUNK_SIZE = 1000
        for i in range(0, len(data), CHUNK_SIZE):
            # print(".", end="")
            yield data[i:i+CHUNK_SIZE]

    tmp = AudioSample(audio_data_wav, force_read_format='s16le', force_sample_rate=8000).as_data(force_out_format='mulaw')
    tmp2 = b''
    au = AudioSample(chunk_data(audio_data_wav), force_sample_rate=8000)
    for i,j in enumerate(au.as_data_stream(force_out_format='mulaw')):
       tmp2 += j
    print(f"tmp: {len(tmp)=}, {len(tmp2)=}")
    assert np.all(AudioSample(tmp2, force_read_format='mulaw').as_numpy() == AudioSample(tmp, force_read_format='mulaw').as_numpy())

def test_stream_numpy_to_mulaw():
    audio_data_np = AudioSample().beep(100).as_numpy()
    def chunk_data_as_numpy(np_data):
        CHUNK_SIZE = 1000
        for i in range(0, np_data.shape[-1], CHUNK_SIZE):
            # print(".", end="")
            yield np_data[..., i:i+CHUNK_SIZE]

    
    chunked = chunk_data_as_numpy(audio_data_np)
    l = 0
    for c in chunked:
        l += c.shape[-1]
    print(f"len: {l=} vs. {audio_data_np.shape[-1]=}")
    assert l == audio_data_np.shape[-1]
    
    tmp = AudioSample(audio_data_np, force_read_sample_rate=48000, force_sample_rate=8000).as_data(force_out_format='mulaw')

    au = AudioSample(chunk_data_as_numpy(audio_data_np), force_read_sample_rate=48000, force_sample_rate=8000)
    tmp2 = b''
    
    for i,j in enumerate(au.as_data_stream(force_out_format='mulaw')):
       tmp2 += j

    assert tmp == tmp2
